const bodyParser = require('body-parser')

const CollectionsUsers = app => {
  app.use(bodyParser.json())
  const basePath = '/api/collections/:collectionId/users'
  const routePath = './routes/collectionsUsers'
  const authBearer = app.locals.passport.authenticate('bearer', {
    session: false,
  })
  /**
   * @api {post} /api/collections/:collectionId/users Add a user to a collection
   * @apiGroup CollectionsUsers
   * @apiParam {collectionId} collectionId Collection id
   * @apiParamExample {json} Body
   *    {
   *      "email": "email@example.com",
   *      "role": "author", [acceptedValues: author],
   *      "isSubmitting": true,
   *      "isCorresponding": false
   *    }
   * @apiSuccessExample {json} Success
   *    HTTP/1.1 200 OK
   *    {
   *      "id": "a6184463-b17a-42f8-b02b-ae1d755cdc6b",
   *      "type": "user",
   *      "admin": false,
   *      "email": "email@example.com",
   *      "teams": [
   *        "c576695a-7cda-4e27-8e9c-31f3a0e9d592"
   *      ],
   *      "username": "email@example.com",
   *      "fragments": [],
   *      "collections": [],
   *      "isConfirmed": false,
   *      "editorInChief": false,
   *      "handlingEditor": false,
   *      "passwordResetToken": "04590a2b7f6c1f37cb84881d529e516fa6fc309c205a07f1341b2bfaa6f2b46c"
   *    }
   * @apiErrorExample {json} Invite user errors
   *    HTTP/1.1 400 Bad Request
   *    HTTP/1.1 500 Internal Server Error
   */
  app.post(
    basePath,
    authBearer,
    require(`${routePath}/post`)(app.locals.models),
  )
  /**
   * @api {delete} /api/collections/:collectionId/user/:userId Delete user from collection
   * @apiGroup CollectionsUsers
   * @apiParam {collectionId} collectionId Collection id
   * @apiParam {userId} userId User id
   * @apiSuccessExample {json} Success
   *    HTTP/1.1 200 {}
   * @apiErrorExample {json} Delete errors
   *    HTTP/1.1 403 Forbidden
   *    HTTP/1.1 404 Not Found
   */
  app.delete(
    `${basePath}/:userId`,
    authBearer,
    require(`${routePath}/delete`)(app.locals.models),
  )
  /**
   * @api {get} /api/collections/:collectionId/users List collections users
   * @apiGroup CollectionsUsers
   * @apiParam {collectionId} collectionId Collection id
   * @apiSuccessExample {json} Success
   *    HTTP/1.1 200 OK
   *    [{
   *      "id": "a6184463-b17a-42f8-b02b-ae1d755cdc6b",
   *      "type": "user",
   *      "admin": false,
   *      "email": "email@example.com",
   *      "teams": [
   *        "c576695a-7cda-4e27-8e9c-31f3a0e9d592"
   *      ],
   *      "username": "email@example.com",
   *      "fragments": [],
   *      "collections": [],
   *      "isConfirmed": false,
   *      "editorInChief": false,
   *      "handlingEditor": false,
   *      "passwordResetToken": "04590a2b7f6c1f37cb84881d529e516fa6fc309c205a07f1341b2bfaa6f2b46c",
   *      "isSubmitting": true,
   *      "isCorresponding": false,
   *    },
   *    {
   *      "id": "a6184463-b17a-42f8-b02b-ae1d755cdc6b",
   *      "type": "user",
   *      "admin": false,
   *      "email": "email2@example.com",
   *      "teams": [
   *        "c576695a-7cda-4e27-8e9c-31f3a0e9d592"
   *      ],
   *      "username": "email2@example.com",
   *      "fragments": [],
   *      "collections": [],
   *      "isConfirmed": true,
   *      "editorInChief": false,
   *      "handlingEditor": false",
   *      "isSubmitting": false,
   *      "isCorresponding": false,
   *    }]
   * @apiErrorExample {json} List errors
   *    HTTP/1.1 400 Bad Request
   *    HTTP/1.1 404 Not Found
   */
  app.get(basePath, authBearer, require(`${routePath}/get`)(app.locals.models))
  /**
   * @api {patch} /api/collections/:collectionId/users/:userId Update a user on a collection
   * @apiGroup CollectionsUsers
   * @apiParam {collectionId} collectionId Collection id
   * @apiParam {userId} userId User id
   * @apiParamExample {json} Body
   *    {
   *      "isSubmitting": false,
   *      "isCorresponding": true,
   *      "firstName": "John",
   *      "lastName": "Smith",
   *      "affiliation": "UCLA"
   *    }
   * @apiSuccessExample {json} Success
   *    HTTP/1.1 200 OK
   *    {
   *      "id": "7e8a77f9-8e5c-4fa3-b717-8df9932df128",
   *      "type": "collection",
   *      "owners": [
   *        {
   *           "id": "69ac1ee9-08a8-4ee6-a57c-c6c8be8d3c4f",
   *           "username": "admin"
   *        }
   *      ],
   *      "authors": [
   *        {
   *          "userId": "a6184463-b17a-42f8-b02b-ae1d755cdc6b",
   *          "isSubmitting": false,
   *          "isCorresponding": true
   *        }
   *      ],
   *      "created": 1522829424474,
   *      "customId": "9424466",
   *      "fragments": [
   *        "c35d0bd8-be03-4c16-b869-bd69796c5a21"
   *      ],
   *      "invitations": [
   *        {
   *          "id": "9043a836-0d49-4b8d-be0b-df39071b5c57",
   *          "hasAnswer": false,
   *          "timestamp": 1522831123430,
   *          "isAccepted": false
   *        },
   *      ]
   *    }
   * @apiErrorExample {json} Update invitations errors
   *    HTTP/1.1 403 Forbidden
   *    HTTP/1.1 400 Bad Request
   *    HTTP/1.1 404 Not Found
   *    HTTP/1.1 500 Internal Server Error
   */
  app.patch(
    `${basePath}/:userId`,
    authBearer,
    require(`${routePath}/patch`)(app.locals.models),
  )
}

module.exports = CollectionsUsers
